==> rasa-x-0.20.5.tar.gz: rasa-x-0.20.5/tests/unit/test_sql_migrations.py <==
import pytest
import sqlalchemy as sa
from alembic.operations import Operations
from alembic.runtime.migration import MigrationContext
from sqlalchemy.orm import Session

import rasax.community.sql_migrations as migrations
from rasax.community import config, utils
from rasax.community.database.schema_migrations.alembic import utils as migration_utils
from rasax.community.services.role_service import RoleService
from rasax.community.services.user_service import UserService, ANNOTATOR, ADMIN, TESTER


@pytest.fixture
def alembic_op(session: Session):
    migration_context = MigrationContext.configure(session.get_bind())

    op = Operations(migration_context)

    # Replace `op` import for the time of the tests.
    old_op = migration_utils.op
    migration_utils.op = op

    yield op

    # Revert replacing of the `op` import
    migration_utils.op = old_op


def test_permission_migration(session: Session, role_service: RoleService):
    new_permission = "advanced.modify.projects.create"
    roles_to_migrate = [ADMIN, ANNOTATOR]

    migrations._migrate_new_permission(session, new_permission, roles_to_migrate)

    for role in roles_to_migrate:
        assert new_permission in role_service.get_role_permissions(role)

    other_roles = [TESTER]
    for role in other_roles:
        assert new_permission not in role_service.get_role_permissions(role)


def test_system_user_exists(session: Session, user_service: UserService):
    # system user should already exist
    assert user_service.fetch_user(config.SYSTEM_USER)

    # delete system user
    existing_user = user_service._fetch_user(config.SYSTEM_USER)
    session.delete(existing_user)
    session.commit()
    assert not user_service.fetch_user(config.SYSTEM_USER)

    # test system user creation
    migrations._create_system_user(session)
    assert user_service.fetch_user(config.SYSTEM_USER)


def test_system_user_creation_community_edition(
    session: Session, user_service: UserService
):
    # delete system user
    existing_user = user_service._fetch_user(config.SYSTEM_USER)
    session.delete(existing_user)
    session.commit()
    assert not user_service.fetch_user(config.SYSTEM_USER)

    # set context to CE
    utils.is_enterprise_installed = lambda: False

    # test system user creation
    migrations._create_system_user(session)
    assert user_service.fetch_user(config.SYSTEM_USER)

    # set context back to EE
    utils.is_enterprise_installed = lambda: True


def test_modify_columns(alembic_op: Operations, session: Session):
    table_name = "test_table"
    test_bool_column = "test_bool_column"
    test_string_column = "test_string_column"

    # Create test table
    alembic_op.create_table(
        table_name,
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column(test_bool_column, sa.Boolean, nullable=False),
        sa.Column(test_string_column, sa.String, nullable=False),
    )

    metadata = sa.MetaData(bind=alembic_op.get_bind())
    metadata.reflect()
    table = sa.Table(table_name, metadata, autoload=True)

    # Insert fake data
    session.execute(table.insert().values((1, False, "some text")))
    session.execute(table.insert().values((2, True, "some other text")))
    session.commit()

    # Assert fake data is there
    entries = session.execute(table.select()).fetchall()
    assert entries == [(1, False, "some text"), (2, True, "some other text")]

    # Transform columns and values in column
    test_bool_transformation = migration_utils.ColumnTransformation(
        test_bool_column,
        new_column_kwargs={"type_": sa.String()},
        modify=lambda x: str(x) + "_test",
    )
    test_string_transformation = migration_utils.ColumnTransformation(
        test_string_column,
        new_column_args=[sa.Text()],
        new_column_kwargs={"nullable": True},
    )

    migration_utils.modify_columns(
        table_name, [test_bool_transformation, test_string_transformation]
    )

    # Update table reflection
    metadata.reflect(extend_existing=True)
    table = sa.Table(table_name, metadata, autoload=True)

    # Test if new value is expected
    entries = session.execute(table.select()).fetchall()

    assert entries == [
        (1, "False_test", "some text"),
        (2, "True_test", "some other text"),
    ]


def test_table_exists(alembic_op: Operations, session: Session):
    table_name = "existing_table"
    test_column = "test_column"

    # Create test table
    alembic_op.create_table(
        table_name,
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column(test_column, sa.String, nullable=False),
    )

    assert migration_utils.table_exists(table_name)
    assert not migration_utils.table_exists("non_existing_table")


def test_table_has_column(alembic_op: Operations, session: Session):
    table_name = "test_column_table"
    test_column = "existing_column"

    # Create test table
    alembic_op.create_table(
        table_name,
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column(test_column, sa.String, nullable=False),
    )

    assert migration_utils.table_has_column(table_name, test_column)
    assert not migration_utils.table_has_column(table_name, "non_existing_column")

==> rasa-x-0.25.0.tar.gz: rasa-x-0.25.0/tests/unit/test_sql_migrations.py <==
import pytest
from alembic.operations import Operations
from alembic.runtime.migration import MigrationContext
from sqlalchemy.orm import Session
import sqlalchemy as sa

import rasax.community.sql_migrations as migrations
from rasax.community import config, utils
from rasax.community.services.role_service import RoleService
from rasax.community.services.user_service import UserService, TESTER
from rasax.community.database.schema_migrations.alembic import utils as migration_utils


@pytest.fixture
def alembic_op(session: Session):
    migration_context = MigrationContext.configure(session.get_bind())

    op = Operations(migration_context)

    # Replace `op` import for the time of the tests.
    old_op = migration_utils.op
    migration_utils.op = op

    yield op

    # Revert replacing of the `op` import
    migration_utils.op = old_op


def test_system_user_exists(session: Session, user_service: UserService):
    # system user should already exist
    assert user_service.fetch_user(config.SYSTEM_USER)

    # delete system user
    existing_user = user_service._fetch_user(config.SYSTEM_USER)
    session.delete(existing_user)
    session.commit()
    assert not user_service.fetch_user(config.SYSTEM_USER)

    # test system user creation
    migrations._create_system_user(session)
    assert user_service.fetch_user(config.SYSTEM_USER)


def test_system_user_creation_community_edition(
    session: Session, user_service: UserService
):
    # delete system user
    existing_user = user_service._fetch_user(config.SYSTEM_USER)
    session.delete(existing_user)
    session.commit()
    assert not user_service.fetch_user(config.SYSTEM_USER)

    # set context to CE
    utils.is_enterprise_installed = lambda: False

    # test system user creation
    migrations._create_system_user(session)
    assert user_service.fetch_user(config.SYSTEM_USER)

    # set context back to EE
    utils.is_enterprise_installed = lambda: True


def test_modify_columns(alembic_op: Operations, session: Session):
    table_name = "test_table"
    test_bool_column = "test_bool_column"
    test_string_column = "test_string_column"

    # Create test table
    alembic_op.create_table(
        table_name,
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column(test_bool_column, sa.Boolean, nullable=False),
        sa.Column(test_string_column, sa.String(255), nullable=False),
    )

    metadata = sa.MetaData(bind=alembic_op.get_bind())
    metadata.reflect()
    table = sa.Table(table_name, metadata, autoload=True)

    # Insert fake data
    session.execute(table.insert().values((1, False, "some text")))
    session.execute(table.insert().values((2, True, "some other text")))
    session.commit()

    # Assert fake data is there
    entries = session.execute(table.select()).fetchall()
    assert entries == [(1, False, "some text"), (2, True, "some other text")]

    # Transform columns and values in column
    test_bool_transformation = migration_utils.ColumnTransformation(
        test_bool_column,
        new_column_kwargs={"type_": sa.String(255)},
        modify_from_column_value=lambda x: str(x) + "_test",
    )
    test_string_transformation = migration_utils.ColumnTransformation(
        test_string_column,
        new_column_args=[sa.Text()],
        new_column_kwargs={"nullable": True},
    )

    migration_utils.modify_columns(
        table_name, [test_bool_transformation, test_string_transformation]
    )

    # Update table reflection
    metadata.reflect(extend_existing=True)
    table = sa.Table(table_name, metadata, autoload=True)

    # Test if new value is expected
    entries = session.execute(table.select()).fetchall()

    assert entries == [
        (1, "False_test", "some text"),
        (2, "True_test", "some other text"),
    ]


def test_existing_table_exists(alembic_op: Operations, session: Session):
    table_name = "existing_table"
    test_column = "test_column"

    # Create test table
    alembic_op.create_table(
        table_name,
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column(test_column, sa.String(255), nullable=False),
    )

    assert migration_utils.table_exists(table_name)


def test_nonexistent_table_doesnt_exist(alembic_op: Operations, session: Session):
    assert not migration_utils.table_exists("nonexistent_table")


def test_table_has_existing_column(alembic_op: Operations, session: Session):
    table_name = "test_column_table"
    test_column = "existing_column"

    # Create test table
    alembic_op.create_table(
        table_name,
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column(test_column, sa.String(255), nullable=False),
    )

    assert migration_utils.table_has_column(table_name, test_column)


def test_table_doesnt_have_nonexistent_column(alembic_op: Operations, session: Session):
    table_name = "test_column_table_2"
    test_column = "existing_column"

    # Create test table
    alembic_op.create_table(
        table_name,
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column(test_column, sa.String(255), nullable=False),
    )

    assert not migration_utils.table_has_column(table_name, "nonexistent_column")


def test_permission_migration(alembic_op: Operations, role_service: RoleService):
    new_permission = "advanced.modify.projects.create"
    role_to_migrate = TESTER

    migration_utils.add_new_permission_to(role_to_migrate, new_permission)

    assert new_permission in role_service.get_role_permissions(role_to_migrate)
